/**
 * @version 0.1
 * @author Majid Ali Khan
 */
package yaes.framework.entity.control.speedcontrol;

import yaes.framework.world.map.Location;
import yaes.framework.world.map.MapHelper;
import yaes.framework.world.map.PlannedPath;
import yaes.framework.world.map.Speed;

/**
 * Follows a fully specified path at a given constant speed.
 */
public class PathFollowAtGivenSpeedControl extends AbstractSpeedControl {
	private final PlannedPath thePath;
	private final double distanceEpsilon = 1.0;
	private int lastCheckPoint;
	private Location nextCheckPoint;
	double timeResolution = 0.0; // speed resolution
	double distanceResolution = 100.0; // assuming 100 meters /pixel

	public PathFollowAtGivenSpeedControl(PlannedPath path, int speed) {
		this.thePath = path;
		lastCheckPoint = 0;
		nextCheckPoint = thePath.getSource();
		this.timeResolution = (distanceResolution / speed);
	}

	@Override
	public void control(double time) {
		// if we are at the destination, then stop.
		final Location currentLocation = controllable.getLocation();
		if (lastCheckPoint == thePath.getLocationList().size()) {
			controllable.controlSpeed(new Speed(0, 0));
			return;
		}
		// identify if we reached the next checkpoint.
		final double distance = MapHelper.distance(currentLocation,
				nextCheckPoint);
		if (distance < distanceEpsilon) {
			// move to next checkpoint
			lastCheckPoint++;
			if (lastCheckPoint < thePath.getLocationList().size()) {
				nextCheckPoint = thePath.getLocationList().get(lastCheckPoint);
				final Speed speed = new Speed(
						(nextCheckPoint.getX() - currentLocation.getX())
								/ timeResolution,
						(nextCheckPoint.getY() - currentLocation.getY())
								/ timeResolution);
				controllable.controlSpeed(speed);
			}
		}
	}
}
