package yaes.framework.world.sensornetwork;

import yaes.framework.agent.ACLMessage;
import yaes.framework.agent.AbstractCommunicatingAgent;

public abstract class AbstractSensorAgent extends AbstractCommunicatingAgent {
	protected SensorNode node;
	private double transmissionRange = 30;
	private double sensorRange = 20;
	private final SensorNetworkWorld sensorWorld;

	/**
	 * Creates an abstract sensor agent for transmission scheduling purposes.
	 */
	public AbstractSensorAgent(String name, SensorNetworkWorld sensorWorld) {
		super(name, sensorWorld);
		this.sensorWorld = sensorWorld;
	}

	/**
	 * Contains the agents behavior
	 */
	public void action() {
		sense();
	}

	public SensorNode getNode() {
		return node;
	}

	/**
	 * @return the sensorRange
	 */
	public double getSensorRange() {
		return sensorRange;
	}

	public SensorNetworkWorld getSensorWorld() {
		return sensorWorld;
	}

	/**
	 * @return the transmissionRange
	 */
	public double getTransmissionRange() {
		return transmissionRange;
	}

	/**
	 * Simulates sensing
	 */
	protected void sense() {
	}

	public void setNode(SensorNode node) {
		this.node = node;
	}

	/**
	 * @param sensorRange
	 *            the sensorRange to set
	 */
	public void setSensorRange(double sensorRange) {
		this.sensorRange = sensorRange;
	}

	/**
	 * @param transmissionRange
	 *            the transmissionRange to set
	 */
	public void setTransmissionRange(double transmissionRange) {
		this.transmissionRange = transmissionRange;
	}

	@Override
	public String toString() {
		String returnValue = "";
		returnValue += getName();
		return returnValue;
	}

	/**
	 * Utility function. All transmissions should go through this function.
	 * While it is not saving a lot of time, it allows to create intermediate
	 * functions.
	 */
	public void transmit(ACLMessage message) {
		getSensorWorld().transmit(getNode(), message);
	}
}
