package yaes.ui.propertysheet;

import java.awt.Color;
import java.awt.Font;
import java.awt.SystemColor;

/**
 * Settings for the property sheet.
 * 
 * @see PropertySheet
 * 
 */
public class PropertySheetSettings {
	public static final int ALWAYS_AS_STRING = 1;
	/** Constant for preferably showing properties as string. */
	public static final int STRING_PREFERRED = 2;
	/** Constant for preferably painting property values. */
	public static final int PAINTING_PREFERRED = 3;
	/** Constant for unsorted sorting mode. */
	public static final int UNSORTED = 0;
	/** Constant for by-name sorting mode. */
	public static final int SORTED_BY_NAMES = 1;
	/** Constant for by-type sorting mode. */
	public static final int SORTED_BY_TYPES = 2;
	/** Property variables. */
	private static int propertyPaintingStyle = PropertySheetSettings.PAINTING_PREFERRED;
	private static boolean plastic = false;
	/** When it's true only writable properties are showen. */
	private static boolean displayWritableOnly = false;
	private static int sortingMode = PropertySheetSettings.SORTED_BY_NAMES;
	// private static Color valueColor = new Color (0, 0, 128);
	// private static Color valueColor = Color.black;
	private static Color propNameFrgndColor = new Color(255, 255, 255);
	private static Color propValueFrgndColor = new Color(0, 0, 128);
	private static Color disabledColor = SystemColor.textInactiveText;
	// private static Color propNameBkgndColor = new Color(204,255,255);
	// private static Color propValueBkgndColor = new Color(255,204,204);
	private static Color propNameBkgndColor = new Color(72, 111, 144);
	// darker blue private static Color propNameBkgndColor = new
	// Color(61,89,132);
	private static Color propValueBkgndColor = new Color(246, 214, 164);
	private static int propNameFontStyle = Font.BOLD;
	private static int propValueFontStyle = Font.PLAIN;
	static PropertySheetSettings propertySheetSettings = new PropertySheetSettings();

	static PropertySheetSettings getDefault() {
		return PropertySheetSettings.propertySheetSettings;
	}

	/*
	 * Gets foreground color of values.
	 */
	public Color getDisabledPropertyColor() {
		if (PropertySheetSettings.disabledColor == null) {
			PropertySheetSettings.disabledColor = SystemColor.textInactiveText;
		}
		return PropertySheetSettings.disabledColor;
	}

	/*
	 * Getter method for visibleWritableOnly property. If is true only writable
	 * properties are showen in propertysheet.
	 */
	public boolean getDisplayWritableOnly() {
		return PropertySheetSettings.displayWritableOnly;
	}

	/*
	 * Returns true if buttons in sheet are plastic.
	 */
	public boolean getPlastic() {
		return PropertySheetSettings.plastic;
	}

	/*
	 * Returns mode of showing properties.
	 * 
	 * @return <CODE>int</CODE> mode of showing properties.
	 * 
	 * @see #setExpert
	 */
	public int getPropertyPaintingStyle() {
		return PropertySheetSettings.propertyPaintingStyle;
	}

	public Color getPropNameBkgndColor() {
		return PropertySheetSettings.propNameBkgndColor;
	}

	/*
	 * Gets font style of property names.
	 */
	public int getPropNameFontStyle() {
		return PropertySheetSettings.propNameFontStyle;
	}

	/*
	 * Gets foreground color of property names.
	 */
	public Color getPropNameFrgndColor() {
		return PropertySheetSettings.propNameFrgndColor;
	}

	public Color getPropValueBkgndColor() {
		return PropertySheetSettings.propValueBkgndColor;
	}

	/*
	 * Gets font style of values.
	 */
	public int getPropValueFontStyle() {
		return PropertySheetSettings.propValueFontStyle;
	}

	/*
	 * Gets foreground color of values.
	 */
	public Color getPropValueFrgndColor() {
		return PropertySheetSettings.propValueFrgndColor;
	}

	/*
	 * Returns sorting mode.
	 * 
	 * @return Sorting mode.
	 */
	public int getSortingMode() {
		return PropertySheetSettings.sortingMode;
	}

	/*
	 * Sets foreground color of disabled property.
	 */
	public void setDisabledPropertyColor(Color color) {
		PropertySheetSettings.disabledColor = color;
	}

	/*
	 * Setter method for visibleWritableOnly property. If is true only writable
	 * properties are showen in propertysheet.
	 */
	public void setDisplayWritableOnly(boolean b) {
		final Boolean oldValue = new Boolean(
				PropertySheetSettings.displayWritableOnly);
		PropertySheetSettings.displayWritableOnly = b;
	}

	/*
	 * Sets buttons in sheet to be plastic.
	 */
	public void setPlastic(boolean plastic) {
		PropertySheetSettings.plastic = plastic;
	}

	/*
	 * Sets property showing mode.
	 */
	public void setPropertyPaintingStyle(int style) {
		PropertySheetSettings.propertyPaintingStyle = style;
	}

	public void setPropNameBkgndColor(Color color) {
		PropertySheetSettings.propNameBkgndColor = color;
	}

	/*
	 * Sets font style of property names.
	 */
	public void setPropNameFontStyle(int style) {
		PropertySheetSettings.propNameFontStyle = style;
	}

	/*
	 * Sets foreground color of property names.
	 */
	public void setPropNameFrgndColor(Color color) {
		PropertySheetSettings.propNameFrgndColor = color;
	}

	public void setPropValueBkgndColor(Color color) {
		PropertySheetSettings.propValueBkgndColor = color;
	}

	/*
	 * Sets font style of values.
	 */
	public void setPropValueFontStyle(int style) {
		PropertySheetSettings.propValueFontStyle = style;
	}

	/*
	 * Sets foreground color of values.
	 */
	public void setPropValueFrgndColor(Color color) {
		PropertySheetSettings.propValueFrgndColor = color;
	}

	/*
	 * Sets sorting mode.
	 * 
	 * @param sortingMode New sorting mode.
	 */
	public void setSortingMode(int sortingMode) {
		PropertySheetSettings.sortingMode = sortingMode;
		// firePropertyChange (PropertySheet.PROPERTY_SORTING_MODE, new Integer
		// (oldValue), new Integer (sortingMode));
	}
}
