/*
 *                 Sun Public License Notice
 *
 * The contents of this file are subject to the Sun Public License
 * Version 1.0 (the "License"). You may not use this file except in
 * compliance with the License. A copy of the License is available at
 * http://www.sun.com/
 *
 * The Original Code is NetBeans. The Initial Developer of the Original
 * Code is Sun Microsystems, Inc. Portions Copyright 1997-2001 Sun
 * Microsystems, Inc. All Rights Reserved.
 */
package yaes.ui.propertysheet;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Rectangle;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.util.Vector;

import javax.swing.FocusManager;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

// import org.openide.awt.MouseUtils;
// import org.openide.util.Utilities;
/**
 * A lightweight component creating a labelled button. Can be "plastic", which
 * means that when a mouse enters the button, the area of the button is stubbed.
 * <code>SheetButton</code> can contain one inner component.
 * 
 * @author Jan Jancura
 */
class SheetButton extends JPanel {
	// innerclasses
	// ..........................................................................
	/** Mouse, key and focus listener. */
	private class IL extends MouseMotionAdapter implements MouseListener,
			KeyListener, FocusListener {
		IL() {
		}

		/** Implements <code>FocusListener</code> interface method. */
		public void focusGained(FocusEvent fe) {
			notifySheetButtonListenersAboutEntered(new ActionEvent(
					SheetButton.this, ActionEvent.ACTION_FIRST + 1,
					actionCommand));
			if (TableSheet.activateEditorFlag) {
				notifySheetButtonListenersAboutClick(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST + 1,
						actionCommand));
				TableSheet.activateEditorFlag = false;
			}
			repaint();
		}

		/** Implements <code>FocusListener</code> interface method. */
		public void focusLost(FocusEvent fe) {
			if (isPlastic) {
				pressed = false;
				setBorder(new EmptyBorder(1, 1, 1, 1));
			}
			notifySheetButtonListenersAboutExited(new ActionEvent(
					SheetButton.this, ActionEvent.ACTION_FIRST + 1,
					actionCommand));
			repaint();
		}

		/** Implements <code>KeyListener</code> interface method. */
		public void keyPressed(KeyEvent e) {
			if (e.isControlDown()) {
				return;
			}
			if (e.getKeyCode() == KeyEvent.VK_E) {
				notifySheetButtonListenersAboutClick(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST + 1, "click"));
			} else if (e.getKeyCode() == KeyEvent.VK_DOWN) {
				final FocusManager fm = FocusManager.getCurrentManager();
				fm.focusNextComponent(SheetButton.this);
			} else {
				if (e.getKeyCode() == KeyEvent.VK_UP) {
					final FocusManager fm = FocusManager.getCurrentManager();
					fm.focusPreviousComponent(SheetButton.this);
				}
			}
		}

		/** Dummy implementation of <code>KeyListener</code> interface method. */
		public void keyReleased(KeyEvent e) {
		}

		/** Implements <code>KeyListener</code> interface method. */
		public void keyTyped(KeyEvent e) {
			if ((e.getKeyChar() == ' ') || (e.getKeyChar() == '\n')) {
				notifySheetButtonListenersAboutClick(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST + 1,
						actionCommand));
			}
		}

		/**
		 * Dummy implementation of <code>MouseListener</code> interface
		 * method.
		 */
		public void mouseClicked(MouseEvent e) {
		}

		/** Overrides superclass method. */
		@Override
		public void mouseDragged(MouseEvent e) {
			if (new Rectangle(SheetButton.this.getSize())
					.contains(e.getPoint()) == pressed) {
				return;
			}
			if (pressed) {
				notifySheetButtonListenersAboutExited(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST,
						actionCommand));
			} else {
				notifySheetButtonListenersAboutEntered(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST,
						actionCommand));
			}
			pressed = !pressed;
			if (pressed) {
				setBorder(new EmptyBorder(2, 2, 0, 0));
			} else {
				setBorder(new EmptyBorder(1, 1, 1, 1));
			}
			if (innerComponent != null) {
				invalidate();
				if (getParent() != null) {
					getParent().validate();
				}
			}
			repaint();
		}

		/** Implements <code>MouseListener</code> interface method. */
		public void mouseEntered(MouseEvent e) {
			if (isPlastic) {
				pressed = true;
				setBorder(new EmptyBorder(2, 2, 0, 0));
				repaint();
				if (plasticNotify) {
					notifySheetButtonListenersAboutEntered(new ActionEvent(
							SheetButton.this, ActionEvent.ACTION_FIRST,
							actionCommand));
				}
			}
		}

		/** Implements <code>MouseExited</code> interface method. */
		public void mouseExited(MouseEvent e) {
			if (isPlastic) {
				pressed = false;
				setBorder(new EmptyBorder(1, 1, 1, 1));
				repaint();
				if (plasticNotify) {
					notifySheetButtonListenersAboutExited(new ActionEvent(
							SheetButton.this, ActionEvent.ACTION_FIRST,
							actionCommand));
				}
			}
		}

		/** Implements <code>MouseListener</code> interface method. */
		public void mousePressed(MouseEvent e) {
			if (!isPlastic) {
				pressed = true;
				setBorder(new EmptyBorder(2, 2, 0, 0));
				notifySheetButtonListenersAboutEntered(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST,
						actionCommand));
				if (innerComponent != null) {
					// setBorder (new EmptyBorder (2, 2, 0, 0));
					invalidate();
					if (getParent() != null) {
						getParent().validate();
					}
					innerComponent.addMouseMotionListener(innerListener);
				} else {
					addMouseMotionListener(innerListener);
				}
				repaint();
			}
		}

		/** Implements <code>MouseListener</code> interface method. */
		public void mouseReleased(MouseEvent e) {
			final int m = e.getModifiers();
			final boolean isRightClick = ((m & InputEvent.BUTTON1_MASK) == 0)
					&& ((m & (InputEvent.BUTTON2_MASK | InputEvent.BUTTON3_MASK)) != 0);
			if (innerComponent != null) {
				innerComponent.removeMouseMotionListener(innerListener);
				// setBorder (new EmptyBorder (1, 1, 1, 1));
				invalidate();
				if (getParent() != null) {
					getParent().validate();
				}
			} else {
				removeMouseMotionListener(innerListener);
			}
			repaint();
			if (isEnabled()) {
				notifySheetButtonListenersAboutClick(new ActionEvent(
						SheetButton.this,
						e.getClickCount() % 2 == 1 ? ActionEvent.ACTION_FIRST + 1
								: ActionEvent.ACTION_FIRST + 2,
						isRightClick ? SheetButton.RIGHT_MOUSE_COMMAND
								: actionCommand));
				notifySheetButtonListenersAboutExited(new ActionEvent(
						SheetButton.this, ActionEvent.ACTION_FIRST,
						actionCommand));
			}
			pressed = false;
			setBorder(new EmptyBorder(1, 1, 1, 1));
		}
	} // End of class IL.

	/*
	 * public javax.accessibility.AccessibleContext getAccessibleContext() { if
	 * (accessibleContext == null) { accessibleContext = new
	 * AccessibleSheetButton(); } return accessibleContext; }
	 * 
	 * class AccessibleSheetButton extends AccessibleJComponent {
	 * 
	 * public javax.accessibility.AccessibleRole getAccessibleRole() { return
	 * javax.accessibility.AccessibleRole.PUSH_BUTTON; }
	 * 
	 * public String getAccessibleName() { String name =
	 * super.getAccessibleName();
	 * 
	 * if (name == null && innerComponent != null) { name =
	 * innerComponent.getAccessibleContext().getAccessibleName(); }
	 * 
	 * if (name == null && label != null) { name = label; }
	 * 
	 * return java.text.MessageFormat.format(
	 * org.openide.util.NbBundle.getBundle(SheetButton.class).getString("ACS_SheetButton"),
	 * new Object[] { new Integer(invalidMark ? 0 : 1), name } ); } }
	 */
	/** generated Serialized Version UID */
	static final long serialVersionUID = -5681433767155127558L;
	/** Action command idicating the right mouse button was clicked. */
	static final String RIGHT_MOUSE_COMMAND = "rightMouseActionCommand"; // NOI18N
	// variables
	// .................................................................
	/** There are all listeners stored. */
	private final transient Vector listeners = new Vector(1, 5);
	/** There are action command string stored. Default value is "click". */
	private String actionCommand = "click"; // NOI18N
	/** Label of this button. */
	private String label;
	/** Preferred size of this button. */
	private Dimension preferredSize;
	/** State of button. */
	private boolean pressed;
	/** True if button can receive focus. */
	private boolean focusTransferable;
	/** True is is in "plastic" mode. */
	private boolean isPlastic;
	/** Listens on focus/key/mouse events. */
	private final IL innerListener;
	/** <code>SheetButton</code> can contain one inner component. */
	private JComponent innerComponent;
	/** Is true if mouseEntered and mouseExited actions should be propagated. */
	private final boolean plasticNotify;
	/** Active foreground color. */
	private Color foreground = SystemColor.controlText;
	/** Inactive foreground color. */
	private Color inForeground = SystemColor.textInactiveText;

	/** Flag indicating invalidity. Invalid mark. */
	private boolean invalidMark;

	// init .................................................................
	/**
	 * Construct a button with empty label.
	 */
	public SheetButton() {
		this("", false, false); // NOI18N
	}

	/**
	 * Construct a button with label and standard appearance (not plastic).
	 * 
	 * @param aLabel
	 *            the label
	 */
	public SheetButton(String aLabel) {
		this(aLabel, false, false);
	}

	/**
	 * Construct a button with label.
	 * 
	 * @param aLabel
	 *            the label
	 * @param boolean
	 *            <code>true</code> if should use plastic appearance
	 */
	public SheetButton(String aLabel, boolean isPlasticButton) {
		this(aLabel, isPlasticButton, false);
	}

	/**
	 * Construct a button with label and possibly a plastic listener.
	 * 
	 * @param label
	 *            the label
	 * @param isPlasticButton
	 *            <code>true</code> if should be plastic
	 * @param plasticActionNotify
	 *            <code>true</code> if mouse enter/exit events should be
	 *            propagated
	 */
	public SheetButton(String aLabel, boolean isPlasticButton,
			boolean plasticActionNotify) {
		setDoubleBuffered(false);
		setOpaque(true);
		label = aLabel == null ? "" : aLabel; // NOI18N
		isPlastic = isPlasticButton;
		plasticNotify = plasticActionNotify;
		innerListener = new IL();
		addMouseListener(innerListener);
		addKeyListener(innerListener);
	}

	/**
	 * Attaches component for this button.
	 */
	@Override
	public Component add(Component innerComponent) {
		setLayout(new BorderLayout());
		setBorder(new EmptyBorder(1, 1, 1, 1));
		add(innerComponent, BorderLayout.CENTER);
		removeMouseListener(innerListener);
		innerComponent.addMouseListener(innerListener);
		return this.innerComponent = (JComponent) innerComponent;
	}

	// SheetButtonListener support
	// ......................................................
	/**
	 * Adds <code>SheetButtonListener</code>.
	 */
	void addSheetButtonListener(SheetButtonListener sheetButtonListener) {
		if (!listeners.contains(sheetButtonListener)) {
			listeners.addElement(sheetButtonListener);
		}
	}

	/**
	 * Getter for <code>foreground</code> property.
	 * 
	 * @return the color
	 */
	public Color getActiveForeground() {
		return foreground;
	}

	/**
	 * Getter for <code>inForeColor</code> property.
	 * 
	 * @return the color of inactive property
	 */
	public Color getInactiveForeground() {
		return inForeground;
	}

	/** Gets invalid image. */
	private Image getInvalidImage() {
		// return
		// Utilities.loadImage("org/openide/resources/propertysheet/invalid.gif");
		// //NOI18N
		return null;
	}

	/**
	 * Gets the label of this button.
	 * 
	 * @return the button's label.
	 */
	public String getLabel() {
		return label;
	}

	/**
	 * Gets preferred size. Overrides superclass method.
	 * 
	 * @return Standart method returned preferredSize (depends on font size
	 *         only).
	 */
	@Override
	public Dimension getPreferredSize() {
		if (preferredSize == null) {
			updatePreferredSize();
		}
		return preferredSize == null ? new Dimension(10, 10) : preferredSize;
	}

	// variables
	// .................................................................
	/**
	 * Identifies whether or not this component can receive the focus. Getter
	 * for <code>focusTransferble</code> property. Overrides superclass
	 * method.
	 * 
	 * @see #focusTransferable
	 */
	@Override
	public boolean isFocusable() {
		return focusTransferable;
	}

	/**
	 * Test whether button is plastic.
	 * 
	 * @return <code>true</code> if so
	 */
	public boolean isPlastic() {
		return isPlastic;
	}

	/**
	 * Test whether button is pressed.
	 * 
	 * @return <code>true</code> if so
	 */
	public boolean isPressed() {
		return pressed;
	}

	/** Notifies listeners button was clicked. */
	public void notifySheetButtonListenersAboutClick(ActionEvent e) {
		final Vector l = (Vector) listeners.clone();
		int i;
		final int k = l.size();
		for (i = 0; i < k; i++) {
			((SheetButtonListener) l.elementAt(i)).sheetButtonClicked(e);
		}
	}

	/** Notifies listeners button was entered. */
	public void notifySheetButtonListenersAboutEntered(ActionEvent e) {
		final Vector l = (Vector) listeners.clone();
		int i;
		final int k = l.size();
		for (i = 0; i < k; i++) {
			((SheetButtonListener) l.elementAt(i)).sheetButtonEntered(e);
		}
	}

	/** Notifies listeners buttoin was exited. */
	public void notifySheetButtonListenersAboutExited(ActionEvent e) {
		final Vector l = (Vector) listeners.clone();
		int i;
		final int k = l.size();
		for (i = 0; i < k; i++) {
			((SheetButtonListener) l.elementAt(i)).sheetButtonExited(e);
		}
	}

	/**
	 * Standart methods painting SheetButton. Overrides superclass method.
	 */
	@Override
	public void paint(Graphics g) {
		super.paint(g);
		final Dimension size = getSize();
		final Color color = g.getColor();
		final Font theFont = g.getFont();
		int xShift = 0;
		if (invalidMark) {
			xShift = getInvalidImage().getWidth(null) + 2;
		}
		g.setFont(getFont());
		final FontMetrics fontMetrics = g.getFontMetrics();
		if (pressed) {
			if (innerComponent == null) {
				g.setColor(isEnabled() ? getActiveForeground()
						: getInactiveForeground());
				g.drawString(label, 6 + xShift, (size.height - fontMetrics
						.getHeight())
						/ 2 + 1 + fontMetrics.getMaxAscent());
				if (invalidMark) {
					g.drawImage(getInvalidImage(), 2, 2, null);
				}
			}
		} else {
			g.setColor(SystemColor.controlLtHighlight);
			g.drawLine(0, 0, size.width - 1, 0);
			g.drawLine(0, 0, 0, size.height - 1);
			g.setColor(SystemColor.controlDkShadow);
			g.drawLine(size.width - 1, 0, size.width - 1, size.height - 1);
			g.drawLine(0, size.height - 1, size.width - 1, size.height - 1);
			if (innerComponent == null) {
				g.setColor(isEnabled() ? getActiveForeground()
						: getInactiveForeground());
				g.drawString(label, 5 + xShift, (size.height - fontMetrics
						.getHeight())
						/ 2 + fontMetrics.getMaxAscent());
				if (invalidMark) {
					g.drawImage(getInvalidImage(), 1, 1, null);
				}
			}
		}
		if (hasFocus()) {
			g.setColor(UIManager.getColor("Button.focus")); // NOI18N
			g.drawRect(2, 1, size.width - 5, size.height - 4);
		}
		g.setFont(theFont);
		g.setColor(color);
	}

	/**
	 * Removes <code>SheetButtonListener</code>.
	 */
	void removeSheetButtonListener(SheetButtonListener sheetButtonListener) {
		listeners.removeElement(sheetButtonListener);
	}

	/**
	 * Set the action command.
	 * 
	 * @param command
	 *            the new command, default is set to "click"
	 */
	public void setActionCommand(String command) {
		actionCommand = command;
	}

	/**
	 * Setter for <code>foreground</code> property.
	 * 
	 * @param color
	 *            the color to set
	 */
	public void setActiveForeground(Color color) {
		foreground = color;
		repaint();
	}

	/**
	 * Sets whether this component can receive the focus. Setter of
	 * <code>focusTransferable</code> property.
	 * 
	 * @param ft
	 *            whether to be able receive the focus
	 * @see #isFocusable
	 */
	@Override
	public void setFocusable(boolean ft) {
		if (isFocusable() == ft) {
			return;
		}
		focusTransferable = ft;
		if (ft) {
			addFocusListener(innerListener);
		} else {
			removeFocusListener(innerListener);
		}
	}

	/**
	 * Sets the font of this component. Overrides superclass method. Adds
	 * updating of preferred size.
	 * 
	 * @param aFont
	 *            The font to become this component's font.
	 */
	@Override
	public void setFont(Font aFont) {
		super.setFont(aFont);
		updatePreferredSize();
	}

	/**
	 * Sets the inactive foreground color.
	 * 
	 * @param color
	 *            the color
	 */
	public void setInactiveForeground(Color color) {
		inForeground = color;
		repaint();
	}

	/** Sets invalid. */
	void setInvalidMark(boolean invalid) {
		invalidMark = invalid;
		repaint();
	}

	/**
	 * Sets the button's label to be the specified string.
	 * 
	 * @param label
	 *            the new label.
	 */
	public void setLabel(String aLabel) {
		label = aLabel;
		if (isShowing()) {
			updatePreferredSize();
			repaint();
		}
	}

	/**
	 * Set whether button is plastic.
	 * 
	 * @param plastic
	 *            <code>true</code> if so
	 */
	public void setPlastic(boolean plastic) {
		this.isPlastic = plastic;
		repaint();
	}

	/**
	 * Set whether button should appear stubbed.
	 * 
	 * @param aPressed
	 *            <code>true</code> if so
	 */
	public void setPressed(boolean aPressed) {
		if (pressed == aPressed) {
			return;
		}
		pressed = aPressed;
		if (pressed) {
			setBorder(new EmptyBorder(2, 2, 0, 0));
		} else {
			setBorder(new EmptyBorder(1, 1, 1, 1));
		}
		repaint();
	}

	/**
	 * Sets tooltip test.
	 */
	@Override
	public void setToolTipText(String str) {
		if (innerComponent != null) {
			innerComponent.setToolTipText(str);
		}
		super.setToolTipText(str);
	}

	/**
	 * Returns string representation of this class.
	 * 
	 * @return <code>String</code> Representation of this class.
	 */
	@Override
	public String toString() {
		return getClass().getName() + "[ \"" + label + "\" ]"; // NOI18N
	}

	/**
	 * Recalculates preferred size.
	 */
	private void updatePreferredSize() {
		final Graphics g = getGraphics();
		if (g == null) {
			return;
		}
		Font font = null;
		if ((font = getFont()) == null) {
			return;
		}
		final FontMetrics fontMetrics = g.getFontMetrics(font);
		preferredSize = new Dimension(fontMetrics.stringWidth(label) + 10,
				fontMetrics.getHeight() + 6);
	}
}
