package yaes.ui.propertysheet.editors;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.beans.PropertyEditorSupport;

import javax.swing.JColorChooser;
import javax.swing.JPanel;

import yaes.ui.propertysheet.ErrorPresenter;

public class ColorEditor extends PropertyEditorSupport {
	public class wgColorChooser extends JPanel implements CustomEditorWidget {
		/**
		 * 
		 */
		private static final long serialVersionUID = 1L;
		private final JColorChooser chooser;

		public wgColorChooser() {
			chooser = new JColorChooser();
			final Dimension dim = chooser.getPreferredSize();
			final int width = dim.width;
			final int height = dim.height;
			final int vpad = 20;
			this.setPreferredSize(new Dimension(width, height + vpad));
			this.add(chooser);
		}

		public void actionOk() {
			ColorEditor.this.setValue(chooser.getColor());
		}

		public String getWidgetTitle() {
			return "Select Color";
		}

		public void setColor(Color color) {
			chooser.setColor(color);
		}

		public void setErrorPresenter(ErrorPresenter errorPresenter) {
		}
	}

	private final wgColorChooser wgChooser;

	public ColorEditor() {
		wgChooser = new wgColorChooser();
	}

	@Override
	public String getAsText() {
		final Color color = (Color) getValue();
		return "" + color.getRed() + "," + color.getGreen() + ","
				+ color.getBlue();
	}

	@Override
	public Component getCustomEditor() {
		return wgChooser;
	}

	@Override
	public String getJavaInitializationString() {
		return "new java.awt.Color(" + getAsText() + ")";
	}

	@Override
	public String[] getTags() {
		return null;
	}

	@Override
	public boolean isPaintable() {
		return true;
	}

	@Override
	public void paintValue(Graphics g, Rectangle rectangle) {
		final Color color1 = g.getColor();
		g.setColor(Color.black);
		final Rectangle rc = new Rectangle(rectangle.width / 16,
				rectangle.height / 2);
		g.drawRect(rectangle.x, rectangle.y + rectangle.height / 2 - rc.height
				/ 2, rc.width, rc.height);
		final Color color = (Color) getValue();
		g.setColor(color);
		g.fillRect(rectangle.x + 1, rectangle.y + rectangle.height / 2
				- rc.height / 2 + 1, rc.width - 1, rc.height - 1);
		g.setColor(color1);
		final String text = "(" + color.getRed() + "," + color.getGreen() + ","
				+ color.getBlue() + ")";
		final FontMetrics f = g.getFontMetrics();
		g.drawString(text, rectangle.x + rc.width + 10, rectangle.height
				- f.getHeight() / 2 + f.getMaxAscent() - 8);
	}

	@Override
	public void setAsText(String s) throws IllegalArgumentException {
		final int i = s.indexOf(44);
		final int j = s.indexOf(44, i + 1);
		if ((i < 0) || (j < 0)) {
			throw new IllegalArgumentException(s);
		}
		try {
			final int k = Integer.parseInt(s.substring(0, i));
			final int l = Integer.parseInt(s.substring(i + 1, j));
			final int i1 = Integer.parseInt(s.substring(j + 1));
			final Color color1 = new Color(k, l, i1);
			setValue(color1);
			wgChooser.setColor(color1);
		} catch (final Exception ex) {
			throw new IllegalArgumentException(s);
		}
	}

	@Override
	public boolean supportsCustomEditor() {
		return true;
	}
}
